/* --COPYRIGHT--,BSD
 * Copyright (c) 2015, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/
 
/*
 * The main goal of this project is implement auto torque feature utilizing MSP430G2553 and DRV8711.
 * The key idea is:
 * monitor the back emf signal from DRV8711--> Detect the loading on motor ---> Change the torque of the motor dynamically
 *  Created on: Aug 28, 2015
 *      Author: a0226371, Preethi Peddagorla
 *      Texas Instruments
 */
#include "msp430g2553.h"
#include "variable_lib.h"

#define nFAULT      0x80               								 // P2.7 nFAULT
#define nSTALL      0x40                							 // P2.6 nSTALL
#define BIN1        0x20                							 // P2.5 PWM AN1
#define BIN2        0x10               								 // P2.4 PWM AN2
#define sCS         0x08                							 // P2.3 SCS
#define AIN2_DIR    0x04                							 // P2.2 DIR
#define AIN1_STEP   0x02               								 // P2.1 STEP
#define RESET       0x01                							 // P2.0 RESET

#define SDATI       0x80               								 // P1.7 SDATI(UCB0SIMO)
#define SDATO       0x40                							 // P1.6 SDATO(UCB0SOMI)
#define SCLK        0x20                							 // P1.5 SCLK (UCB0CLK)
#define nSLEEP      0x10                							 // P1.4 nSLEEP
#define PUS_BUT     0x08                							 // P1.3 Direction select
#define P12         0x04               								 // P1.2 LED ouput
#define P11         0x02                							 // P1.1 back emf input
#define POT         0x01               								 // P1.0 Ref Speed input

//Intialization of variables

unsigned int SPEEDRXD;                  							// user speed received
unsigned int SPEEDTXD;                 				    			// transmitted speed - countdown buffer
volatile int Vbemf[8];
volatile int Speed_present;
volatile short int Speed_ref;
volatile signed int count;               							// for ADC initialization using software trigger
volatile int Speed_error;
volatile unsigned char control;
volatile float control_1;
volatile float integral;
volatile int adc[2];
volatile int Speed_newref_avg;
volatile int Speed_newref;
volatile int Vbemf_avg;
int i;
int j;

// Function prototypes
void Init_Sys(void);                   								 // Initalize System and Module
void Adc_init(void);
void drv8711_init(void);

void main(void) {
	// Initialize  system
 	Init_Sys();
	Adc_init();
	drv8711_init();
	_BIS_SR(GIE);                          						    // Enable global interrupt

	//Starting of recurring  loop

	while (1) {

		if (count == 5)                           				    //bemf signal is sampled for every fifth peak transition of STALLn pin
				{
			count = 0;
			__delay_cycles(200);                                     //delay for adc before starting to sample
			ADC10CTL0 &= ~ENC;                                 	     // Enabling ADC
			while (ADC10CTL1 & BUSY);                          	     // Wait if ADC10 core is active
			ADC10SA = (unsigned int) adc; 							// Copies data in ADC10SA to unsigned int adc array
			ADC10CTL0 |= ENC + ADC10SC;         					 // Sampling and conversion start

			Vbemf[i] = adc[0] / 8;									// adc array 0 copied to Vbemf array after reducing the resolution from 10 to 7
			Speed_newref = adc[1] / 32;	                            // adc array 1 copied to Speed_newref after reducing the resolution from 10 to 5
			i = i + 1;
			if (i == 8) {                                           // To obtain 8 samples of bemf to avoid rapid fluctuations

				Speed_ref = Speed_functn[Speed_newref]; 			//Speed reference obtained from look up table

				if (Speed_ref < Speed_min) {					// motor stops
									P2OUT |= sCS;                   // SPI open
									UCB0TXBUF = 0x0E; 				// CTRL = Enable motor + default+ ISgain-5
									UCB0TXBUF = 0x90;				//
									while (UCB0STAT & BUSY);         // USCI_B0 complete?
									P2OUT &= ~sCS;                   // SPI closed

									P1OUT &= ~BIT2;					//LED1 turned off
									j = 1;							//To detect the state of the motor
									i = 7;							//Inorder to ensure continuous operation of main loop
									count = 5;


				}
				else if (Speed_ref < Speed_min							//motor runs under no control
						&& Speed_ref >= Speed_max_NC) {
					TA1CCR0 = timer_period_new[Speed_newref] - 1; 	// timer period value corresponding to a given speed ref obtained from look up table
					P1OUT |= BIT2;									//LED1 turned on
					if (j == 1) {                                   //motor turns on if it is at rest
						P2OUT |= sCS;                               // SPI open
						UCB0TXBUF = 0x0E; 							// CTRL = Enable motor + default+ ISgain-5
						UCB0TXBUF = 0x91;
						while (UCB0STAT & BUSY);                   // USCI_B0 complete?
						P2OUT &= ~sCS;                             // SPI closed
						j = 0;									   // To detect the state of the motor
					}
					P2OUT |= sCS;                         		   // SPI open
					UCB0TXBUF = 0x13;               			   // Adjusting torque register
					UCB0TXBUF = 0xB4;							   // Giving reasonably large value to have high chopping current
					while (UCB0STAT & BUSY); 		               // USCI_B0 complete?
					P2OUT &= ~sCS; 									// SPI closed

					i = 7;										  //Inorder to ensure continuous operation of main loop
					count = 4;

				}

				else {
					P1OUT &= ~BIT2;
					if (j == 1) { 								  //motor turns on if it is at rest
						P2OUT |= sCS;                             // SPI open
						UCB0TXBUF = 0x0E;                         // CTRL = Enable motor + default+ ISgain-5
						UCB0TXBUF = 0x91;
						while (UCB0STAT & BUSY); 	              // USCI_B0 complete?
						P2OUT &= ~sCS;                            // SPI closed
						j = 0;									  // To detect the state of the motor
					}

					Vbemf_avg = (Vbemf[0] + Vbemf[1] + Vbemf[2] + Vbemf[3]
							+ Vbemf[4] + Vbemf[5] + Vbemf[6] + Vbemf[7]) / 8;     // Averaging the bemf samples
					Speed_present = speed_4m_bemf[Vbemf_avg];      // present speed of motor obtained from look up table

					TA1CCR0 = timer_period_new[Speed_newref] - 1;  // timer period value corresponding to a given speed ref obtained from look up table
					Speed_error = Speed_ref - Speed_present;

					// To avoid rapid current change, Speed_error is manipulated
					if (Speed_error <= 20 && Speed_error >= 10) {
						Speed_error = 10;
					}
					else if (Speed_error <= 0) {
						Speed_error = 0;                            //Speed_error cannot less than zero since steppers always follow the input pulses
					}
					else if (Speed_error <= 10 && Speed_error > 0) {
						Speed_error = 5;
					}

					//Manipulating the integral value to attain reduction in torque values when operated at a steady load
					if (integral > 300)
						integral = 300;
					else if (integral> torque_min && integral <= 300)
					    integral-=10;
					if (integral < torque_min)
						integral = torque_min;

					//PI controller
					integral = integral + Speed_error * ki;
					control_1 = Speed_error * kp + integral;

					//Manipulating control_1 to maintain the minimum and maximum constraints
					if (control_1 >= 255)
						control = 255;
					else if (control_1 <= torque_min)
						control = torque_min;
					else
						control = control_1;

					// Motor control

					P2OUT |= sCS;                                 // SPI open
					UCB0TXBUF = 0x13;           		          // Adjusting torque register
					UCB0TXBUF = control;
					while (UCB0STAT & BUSY);                      // USCI_B0 complete?
					P2OUT &= ~sCS;                                // SPI closed
					i = 0;

				}
			}

		}

	}

}

/*------------------------------------------------------------------------------
 Initalize System and Module
 ------------------------------------------------------------------------------*/
void Init_Sys(void) {
	__bis_SR_register(OSCOFF);            						// XTAL not used
	WDTCTL = WDTPW + WDTHOLD;            					    // Stop watchdog timer
	BCSCTL1 = CALBC1_8MHZ;    									//  Set range
	DCOCTL = CALDCO_8MHZ;    									//  Set DCO step    +   modulation
	count = 0;                  								// initalization for software trigger of ADC

	//Initialization of variables
	Speed_ref = Speed_max;
	integral = 0;
	Speed_error = 0;
	Speed_present = 0;
	control = 0;
	j = 1;

	//Initialization of SPI
	P1SEL = SDATI + SDATO + SCLK;          						// SPI, overrides direction
	P1SEL2 = SDATI + SDATO + SCLK;        						// SPI
	P1DIR = nSLEEP;                        						// P1.x outputs
	P2DIR = sCS + AIN2_DIR;    									// P2.x outputs
	CAPD = CAPD0 + CAPD1;

	// Setting external interrupt at rising edge of P2.6 input
	P2IE |= SDATO;     											// Interrupt enable from low to high for Stalln input
	P2SEL &= ~SDATO;
	P2SEL2 &= ~SDATO;
	P2DIR &= ~SDATO;

	P1DIR |= BIT2;                                                // Setting P1.2 as output for LED1

    //timer initialization
	P2DIR |= BIT1;
	P2SEL |= BIT1;
	TA1CCR0 |= 300 - 1;
	TA1CCTL1 |= OUTMOD_7;
	TA1CCR1 |= 15;
	TA1CTL = TASSEL_2 + MC_1 + ID_3;              				  // SCLK source, up mode

    //Starting of SPI
	UCB0CTL0 |= UCCKPH + UCMSB + UCMST + UCSYNC;  				  // 3-pin, 8-bit SPI master
	UCB0CTL1 |= UCSSEL_2;                 						  // SMCLK
	UCB0CTL1 &= ~UCSWRST;                 						  // Initialize USCI state machine
	SPEEDTXD = 1;                        						  // No speed
	P1OUT |= nSLEEP;                      						  // Exit sleep

}

void Adc_init(void) {
	__delay_cycles(50);                							 // Wait for ADC Ref to settle
	ADC10CTL1 = INCH_1 + ADC10DIV_0 + CONSEQ_1;
	ADC10CTL0 = SREF_1 + REF2_5V + ADC10SHT_2 + ADC10ON + REFON + MSC;
	ADC10DTC1 = 0x02;
	ADC10AE0 |= BIT1 + BIT0;
}

void drv8711_init(void) {

	// Use SPI to configure DRV8711
	P2OUT |= sCS;                        						 // SPI open
	UCB0TXBUF = 0x0E;             								 // CTRL = Enable motor + default+ ISgain-5
	UCB0TXBUF = 0x91;
	while (UCB0STAT & BUSY);             						 // USCI_B0 complete?
	P2OUT &= ~sCS;                        						 // SPI closed

	P2OUT |= sCS;                        						 // SPI open
	UCB0TXBUF = 0x10;                    						 // TORQUE Register setting
	UCB0TXBUF = 0x50;
	while (UCB0STAT & BUSY);             						 // USCI_B0 complete?
	P2OUT &= ~sCS;                        						// SPI closed

	P2OUT |= sCS;                        					    // SPI open
	UCB0TXBUF = 0x45;                     						// DECAY (auto mixed decay) Register setting
	UCB0TXBUF = 0x10;
	while (UCB0STAT & BUSY);   						           // USCI_B0 complete?
	P2OUT &= ~sCS;           					               // SPI closed

	P2OUT |= sCS;                    					       // SPI open
	UCB0TXBUF = 0x5C;                    					   // STALL Register setting
	UCB0TXBUF = 0xFF;
	while (UCB0STAT & BUSY);  					              // USCI_B0 complete?
	P2OUT &= ~sCS;                       					  // SPI closed
}

#pragma vector = PORT2_VECTOR
__interrupt
void InterruptVectorPort2() {
	if (count > 5)
		count = 0;
	count += 1;
	P2IFG &= ~0x40; 											// Clear Interrupt Flag
}

